# Solitude is a interactive story with four endings.
# Copyright (C) 2012 Ryan Martin
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os.path
import sys
import json

import pyglet
from pyglet.gl import *
from pyglet import sprite
from pyglet import image
from pyglet import resource

from Viewport import FixedResolutionViewport

import GameData
import Interface
from Character import Character
from Entity import Entity
from Stage import Stage

class Farmhouse(Stage):
    def __init__(self):
        width, height=self.load_map("farmhouse.json")
        Stage.__init__(self, width, height)
        self.background=sprite.Sprite(resource.image("farmhouse_background.png"), batch=self.batch, group=self.backgroup)
        self.player.load_walkcycle(resource.image("farm_walkcycle.png"))
        self.player.set_idle("down")
        self.player.x=11*32
        self.player.y=9*32
        self.set_group(self.player)
        self.update_camera()
        self.stove=sprite.Sprite(resource.image("stove_fire.png"), batch=self.batch, group=self.ogroups[1])
        self.stove.x=2*32
        self.stove.y=6*32
        self.set_group(self.stove)
        self.set_action(2*32, 6*32, lambda: self.show_dialog(["The fire is burning but I feel nothing."]))
        self.set_action(1*32, 5*32, lambda: self.show_dialog(["I got rid of the other chair.", "I didn't need it anymore."]))
        self.bookcase=sprite.Sprite(resource.image("bookcase.png"), batch=self.batch, group=self.ogroups[1])
        self.bookcase.x=8*32
        self.bookcase.y=9*32
        self.set_group(self.bookcase)
        self.set_action(8*32, 9*32, lambda: self.show_dialog(["I don't want to read."]))
        self.set_action(8*32, 10*32, lambda: self.show_dialog(["I don't want to read."]))
        self.log=sprite.Sprite(resource.image("log.png"), batch=self.batch, group=self.ogroups[1])
        self.log.x=1*32
        self.log.y=6*32
        self.set_group(self.log)
        self.paper=sprite.Sprite(resource.image("paper.png"), batch=self.batch, group=self.ogroups[1])
        self.paper.x=4*32
        self.paper.y=2*32
        self.set_group(self.paper)
        self.paper.visible=False
        for i,j in self.bed_locations:
            self.set_action(i*32, j*32, lambda text="Sleep?", action=self.sleep: self.show_yes_no_dialog(text, action))
        self.set_tile_action(5, 1, lambda: self.show_dialog("I don't feel like going outside."))
        self.slp=0
        self.is_dream=True
        if Interface.music:
            self.music=resource.media("farmhouse.ogg")

    def sleep(self):
        if self.slp==0:
            self.slp=1
            self.sleep1()
        elif self.slp==1:
            self.slp=2
            self.sleep2()
        else:
            self.show_dialog("I don't want to sleep anymore.")
        
    def sleep1(self):
        self.bookcase.visible=False
        self.log.visible=False
        self.set_action(8*32, 9*32, lambda: self.show_dialog(["Burned when the wood ran out.", "The journal was first."]))
        self.set_action(8*32, 10*32, lambda: self.show_dialog(["Burned when the wood ran out.", "The journal was first."]))
        self.stove.image=resource.image("stove.png")
        self.set_action(2*32, 6*32, lambda: self.show_dialog("Ran out of fuel."))
        self.background.image=resource.image("farmhouse_background2.png")

    def sleep2(self):
        self.paper.visible=True
        self.set_action(4*32, 2*32, lambda: self.show_dialog("A newspaper clipping of Jake and Astraea's wedding announcement."))
        self.unset_tile_action(5, 1) 
        self.unset_collision(5*32, 0*32)
        self.set_tile_action(5, 0, self.end_stage)
        self.background.image=resource.image("farmhouse_background3.png")

    def end_stage(self):
        self.fade_out(lambda: self.show_dialog(["There was nothing left for me there.", "I didn't look back."], self.window.close))

    def load_map(self, mapname):
        width, height=Stage.load_map(self, mapname)
        for layer in self.layers:
            if layer["name"]=="Bed":
                self.bed_locations=self.get_locations(layer['data'], width, height)
        return width, height
