# Solitude is a interactive story with four endings.
# Copyright (C) 2012 Ryan Martin
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os.path
import sys
import json

import pyglet
from pyglet.gl import *
from pyglet import sprite
from pyglet import image
from pyglet import resource

from Viewport import FixedResolutionViewport

import GameData
import Interface
from Character import Character
from Entity import Entity
from Stage import Stage

class Mansion(Stage):
    def __init__(self):
        width, height=self.load_map("mansion.json")
        Stage.__init__(self, width, height)
        self.background=sprite.Sprite(resource.image("mansion_background.png"), batch=self.batch, group=self.backgroup)
        self.building=self.new_sprite(7, 18, resource.image("mansion_building.png"))
        self.building.image.anchor_y=32
        self.building.visible=False
        self.player.load_walkcycle(resource.image("city_walkcycle.png"))
        self.player.set_idle("down")
        self.player.x=11*32+16
        self.player.y=34*32
        self.set_group(self.player)
        self.update_camera()
        cook_walkcycle=image.TextureGrid(image.ImageGrid(resource.image("cook.png"), 4, 9))
        self.cook=self.new_sprite(24, 33, cook_walkcycle[(1,0)])
        self.cook.x-=16
        self.set_collision(24*32, 33*32)
        self.set_action(24*32, 33*32, lambda: self.show_dialog("Just let me know what you want for dinner and I'll whip it right up."))
        self.paper=self.new_sprite(11, 22, resource.image("paper.png"))
        self.set_action(11*32, 21*32, lambda: self.show_dialog(["Now that Player Corp has completed its IPO, Player is one of the richest men in America.",
                                                                "Many wonder what challenge he will take up next."]))
        assistant_walkcycle=image.TextureGrid(image.ImageGrid(resource.image("coworker.png"), 4, 9))
        self.assistant=self.new_sprite(15, 24, assistant_walkcycle[(1,0)])
        self.assistant.x-=16
        self.set_collision(15*32, 24*32)
        self.set_action(15*32, 24*32, lambda: self.show_dialog(["You have an dinner party invitation from Miss Oldmoney.",  
                                                               "Also, Mr. Bigshot would like you to call him at your earliest convenience."]))
        tree_image=resource.image("tree.png")
        tree_image.anchor_x=tree_image.width/2
        self.trees=[]
        for i,j in self.tree_locations:
            tree=self.new_sprite(i, j, tree_image)
            tree.x+=16
            self.trees.append(tree)
        pot_image=resource.image("pot.png")
        self.pots=[]
        for i,j in self.pot_locations:
            pot=self.new_sprite(i, j, pot_image)
            self.pots.append(pot)
        for i,j in self.car_locations:
            self.set_action(i*32, j*32, self.end_stage)
        self.columns1=self.new_sprite(49, 31, resource.image("columns1.png"))
        self.columns2=self.new_sprite(49, 38, resource.image("columns2.png"))
        self.set_tile_action(18, 18, self.hide_building)
        self.set_tile_action(18, 17, self.show_building)
        self.is_dream=True
        if Interface.music:
            self.music=resource.media("mansion.ogg")

    def end_stage(self):
        self.fade_out(lambda: self.show_dialog(["The tank was full.", "I decided to see where it would take me."], self.window.close))

    def show_building(self):
        self.building.visible=True

    def hide_building(self):
        self.building.visible=False

    def load_map(self, mapname):
        width, height=Stage.load_map(self, mapname)
        for layer in self.layers:
            if layer["name"]=="Trees":
                self.tree_locations=self.get_locations(layer['data'], width, height)
            if layer["name"]=="Pots":
                self.pot_locations=self.get_locations(layer['data'], width, height)
            if layer["name"]=="Car":
                self.car_locations=self.get_locations(layer['data'], width, height)
        return width, height
